@@ -2,9 +2,10 @@ import numpy as np
 from numpy.linalg import LinAlgError, inv, pinv
 from numpy import dot
 import pandas as pd
+import pdb
 
 from lifelines.plotting import plot_dataframes
-from lifelines.utils import survival_table_from_events, basis, inv_normal_cdf, quadrature
+from lifelines.utils import survival_table_from_events, basis, inv_normal_cdf, quadrature, epanechnikov_kernel
 
 class NelsonAalenFitter(object):
     """
@@ -44,7 +45,7 @@ class NelsonAalenFitter(object):
         """
         
         if censorship is None:
-           self.censorship = np.ones_like(event_times, dtype=bool) #why boolean?
+           self.censorship = np.ones(event_times.shape[0], dtype=bool) #why boolean?
         else:
            self.censorship = np.array(censorship).copy().astype(bool)
         self.event_times = survival_table_from_events(event_times, self.censorship)
@@ -64,6 +65,9 @@ class NelsonAalenFitter(object):
         self.cumulative_hazard_ = pd.DataFrame(cumulative_hazard_, columns=columns)
         self.confidence_interval_ = self._bounds(cumulative_sq_[:,None],alpha)
         self.plot = plot_dataframes(self, "cumulative_hazard_")
+        self.plot_cumulative_hazard = self.plot
+        self.plot_hazard = plot_dataframes(self, 'hazard_')
+        self._cumulative_sq = cumulative_sq_
 
         return self
 
@@ -90,6 +94,40 @@ class NelsonAalenFitter(object):
        #check it 0
        return (1.*deaths/population).replace([np.inf],0)
 
+    def smoothed_hazard_(self, bandwidth):
+        """
+        bandwidth: the bandwith used in the Epanechnikov kernel.
+
+        """
+        timeline = self.timeline
+        cumulative_hazard_name =  self.cumulative_hazard_.columns[0]
+        hazard_name = "smoothed-" + cumulative_hazard_name
+        hazard_ = self.cumulative_hazard_.diff().fillna(self.cumulative_hazard_.iloc[0] )
+        C = (hazard_[cumulative_hazard_name] != 0.0).values
+        return pd.DataFrame( 1./(2*bandwidth)*np.dot(epanechnikov_kernel(timeline[:,None], timeline[C][None,:],bandwidth), hazard_.values[C,:]), 
+                columns=[hazard_name], index=timeline)
+
+
+    def smoothed_hazard_confidence_intervals_(self, bandwidth, hazard_=None):
+        """
+        bandwidth: the bandwith to use in the Epanechnikov kernel.
+        hazard_: a computed (n,) numpy array of estimated hazard rates. If none, uses naf.smoothed_hazard_
+        """
+        if hazard_==None:
+          hazard_ = self.smoothed_hazard_(bandwidth).values[:,0]
+
+        timeline = self.timeline
+        alpha2 = inv_normal_cdf(1 - (1-self.alpha)/2)
+        name = "smoothed-" + self.cumulative_hazard_.columns[0]
+        self._cumulative_sq.iloc[0] = 0 #why?
+        var_hazard_ = self._cumulative_sq.diff().fillna( self._cumulative_sq.iloc[0] )
+        C = (var_hazard_.values != 0.0) #only consider the points with jumps
+        std_hazard_ = np.sqrt(1./(2*bandwidth**2)*np.dot(epanechnikov_kernel(timeline[:,None], timeline[C][None,:],bandwidth)**2, var_hazard_.values[C]))
+        values = {
+              "%s_upper_%.2f"%(name,self.alpha):hazard_*np.exp(alpha2*std_hazard_/hazard_),
+              "%s_lower_%.2f"%(name,self.alpha):hazard_*np.exp(-alpha2*std_hazard_/hazard_)
+              }
+        return pd.DataFrame(values, index=timeline)
 
 class KaplanMeierFitter(object):
    
@@ -114,7 +152,7 @@ class KaplanMeierFitter(object):
        """
        #set to all observed if censorship is none
        if censorship is None:
-          self.censorship = np.ones_like(event_times, dtype=bool) #why boolean?
+          self.censorship = np.ones(event_times.shape[0], dtype=bool) #why boolean?
        else:
           self.censorship = np.array(censorship).copy()
 
@@ -137,6 +175,8 @@ class KaplanMeierFitter(object):
        #self.median_ = median_survival_times(self.survival_function_)
        self.confidence_interval_ = self._bounds(cumulative_sq_[:,None],alpha)
        self.plot = plot_dataframes(self, "survival_function_")
+       self.plot_survival_function = self.plot
+       self.median_ = median_survival_times(self.survival_function_)
        return self
 
   def _bounds(self, cumulative_sq_, alpha):
@@ -157,6 +197,10 @@ class KaplanMeierFitter(object):
 
 
 def _additive_estimate(events, timeline, _additive_f, _additive_var):
+    """
+    Called to compute the Kaplan Meier and Nelson-Aalen estimates.
+
+    """
     N = events["removed"].sum()
 
     deaths = events['observed']
@@ -170,130 +214,131 @@ def _additive_estimate(events, timeline, _additive_f, _additive_var):
 
 
 class AalenAdditiveFitter(object):
-  """
-  This class fits the regression model:
+    """
+    This class fits the regression model:
 
-  hazard(t)  = b_0(t) + b_t(t)*x_1 + ... + b_N(t)*x_N
+    hazard(t)  = b_0(t) + b_t(t)*x_1 + ... + b_N(t)*x_N
 
-  that is, the hazard rate is a linear function of the covariates. Currently the covariates
-  must be time independent. 
+    that is, the hazard rate is a linear function of the covariates. Currently the covariates
+    must be time independent. 
 
-  Parameters:
-    fit_intercept: If False, do not attach an intercept (column of ones) to the covariate matrix. The 
-      intercept, b_0(t) acts as a baseline hazard. 
-    alpha: the level in the confidence intervals.
-    penalizer: Attach a L2 penalizer to the regression. This improves stability of the estimates
-     and controls high correlation between covariates.
-
-  """
-
-  def __init__(self,fit_intercept=True, alpha=0.95, penalizer = 0.0):
-    self.fit_intercept = fit_intercept
-    self.alpha = alpha
-    self.penalizer = penalizer
-    assert penalizer >= 0, "penalizer keyword must be >= 0."
-
-  def fit(self, event_times, X, timeline = None, censorship=None, columns=None):
-    """currently X is a static (n,d) array
-
-    event_times: (n,1) array of event times
-    X: (n,d) the design matrix, either a numpy matrix or DataFrame.  
-    timeline: (t,1) timepoints in ascending order
-    censorship: (n,1) boolean array of censorships: True if observed, False if right-censored.
-                By default, assuming all are observed.
-
-    Fits: self.cumulative_hazards_: a (t,d+1) dataframe of cumulative hazard coefficients
-          self.hazards_: a (t,d+1) dataframe of hazard coefficients
+    Parameters:
+      fit_intercept: If False, do not attach an intercept (column of ones) to the covariate matrix. The 
+        intercept, b_0(t) acts as a baseline hazard. 
+      alpha: the level in the confidence intervals.
+      penalizer: Attach a L2 penalizer to the regression. This improves stability of the estimates
+       and controls high correlation between covariates.
 
     """
-    #deal with the covariate matrix. Check if it is a dataframe or numpy array
-    n,d = X.shape
-    if type(X)==pd.core.frame.DataFrame:
-      X_ = X.values.copy()
-      if columns is None:
-        columns = X.columns
-    else:
-      X_ = X.copy()
 
-    # append a columns of ones for the baseline hazard
-    ix = event_times.argsort(0)[:,0].copy()
-    X_ = X_[ix,:].copy() if not self.fit_intercept else np.c_[ X_[ix,:].copy(), np.ones((n,1)) ]
-    sorted_event_times = event_times[ix,0].copy()
+    def __init__(self,fit_intercept=True, alpha=0.95, penalizer = 0.0):
+        self.fit_intercept = fit_intercept
+        self.alpha = alpha
+        self.penalizer = penalizer
+        assert penalizer >= 0, "penalizer keyword must be >= 0."
 
-    #set the column's names of the dataframe.
-    if columns is None:
-      columns = range(d) 
-    else:
-      columns = [c for c in columns] 
+    def fit(self, event_times, X, timeline=None, censorship=None, columns=None):
+        """currently X is a static (n,d) array
 
-    if self.fit_intercept:
-      columns += ['baseline']
+        event_times: (n,1) array of event times
+        X: (n,d) the design matrix, either a numpy matrix or DataFrame.  
+        timeline: (t,1) timepoints in ascending order
+        censorship: (n,1) boolean array of censorships: True if observed, False if right-censored.
+                    By default, assuming all are observed.
 
-    #set the censorship events. 1 if the death was observed.
-    if censorship is None:
-        observed = np.ones(n, dtype=bool)
-    else:
-        observed = censorship[ix].reshape(n)
+        Fits: self.cumulative_hazards_: a (t,d+1) dataframe of cumulative hazard coefficients
+              self.hazards_: a (t,d+1) dataframe of hazard coefficients
 
-    #set the timeline -- this is used as DataFrame index in the results
-    if timeline is None:
-        timeline = sorted_event_times.copy()
-
-    timeline = np.unique(timeline.astype(float))
-    if timeline[0] > 0:
-       timeline = np.insert(timeline,0,0.)
-    
-    unique_times = np.unique(timeline)
-    zeros = np.zeros((timeline.shape[0],d+self.fit_intercept))
-    self.cumulative_hazards_ = pd.DataFrame(zeros.copy() , index=unique_times, columns = columns)
-    self.hazards_ = pd.DataFrame(np.zeros((event_times.shape[0],d+self.fit_intercept)), index=event_times[:,0], columns = columns)
-    self._variance = pd.DataFrame(zeros.copy(), index=unique_times, columns = columns)
-    
-    #create the penalizer matrix for L2 regression
-    penalizer = self.penalizer*np.eye(d + self.fit_intercept)
-    
-    t_0 = sorted_event_times[0]
-    cum_v = np.zeros((d+self.fit_intercept,1))
-    v = cum_v.copy()
-    for i,time in enumerate(sorted_event_times):
-        relevant_times = (t_0<timeline)*(timeline<=time)
-        if observed[i] == 0:
-            X_[i,:] = 0
-        try:
-            V = dot(inv(dot(X_.T,X_) + penalizer), X_.T)
-        except LinAlgError:
-            #if penalizer > 0, this should not occur. But sometimes it does...
-            V = dot(pinv(dot(X_.T,X_) + penalizer), X_.T)
-
-        v = dot(V, basis(n,i))
-        cum_v = cum_v + v
-        self.cumulative_hazards_.ix[relevant_times] = self.cumulative_hazards_.ix[relevant_times].values + cum_v.T
-        self.hazards_.iloc[i] = self.hazards_.iloc[i].values + v.T
-        self._variance.ix[relevant_times] = self._variance.ix[relevant_times].values + dot( V[:,i][:,None], V[:,i][None,:] ).diagonal()
-        t_0 = time
-        X_[i,:] = 0
-
-    #clean up last iteration
-    relevant_times = (timeline>time)
-    self.cumulative_hazards_.ix[relevant_times] = cum_v.T
-    self.hazards_.iloc[i] = v.T
-    self._variance.ix[relevant_times] = dot( V[:,i][:,None], V[:,i][None,:] ).diagonal()
-    self.timeline = timeline
-    self.X = X
-    self.censorship = censorship
-    self._compute_confidence_intervals()
-    return self
-
-  def smoothed_hazards_(self, bandwith=1):
         """
-        Using the gaussian kernel to smooth the hazard function, with sigma/bandwith
+        #deal with the covariate matrix. Check if it is a dataframe or numpy array
+        n,d = X.shape
+        if type(X)==pd.core.frame.DataFrame:
+          X_ = X.values.copy()
+          if columns is None:
+            columns = X.columns
+        else:
+          X_ = X.copy()
+
+        # append a columns of ones for the baseline hazard
+        ix = event_times.argsort(0)[:,0].copy()
+        X_ = X_[ix,:].copy() if not self.fit_intercept else np.c_[ X_[ix,:].copy(), np.ones((n,1)) ]
+        sorted_event_times = event_times[ix,0].copy()
+
+        #set the column's names of the dataframe.
+        if columns is None:
+          columns = range(d) 
+        else:
+          columns = [c for c in columns] 
+
+        if self.fit_intercept:
+          columns += ['baseline']
+
+        #set the censorship events. 1 if the death was observed.
+        if censorship is None:
+            observed = np.ones(n, dtype=bool)
+        else:
+            observed = censorship[ix].reshape(n)
+
+        #set the timeline -- this is used as DataFrame index in the results
+        if timeline is None:
+            timeline = sorted_event_times.copy()
+
+        timeline = np.unique(timeline.astype(float))
+        if timeline[0] > 0:
+           timeline = np.insert(timeline,0,0.)
+        
+        unique_times = np.unique(timeline)
+        zeros = np.zeros((timeline.shape[0],d+self.fit_intercept))
+        self.cumulative_hazards_ = pd.DataFrame(zeros.copy(), index=unique_times, columns = columns)
+        self.hazards_ = pd.DataFrame(np.zeros((event_times.shape[0],d+self.fit_intercept)), index=event_times[:,0], columns = columns)
+        self._variance = pd.DataFrame(zeros.copy(), index=unique_times, columns = columns)
+        
+        #create the penalizer matrix for L2 regression
+        penalizer = self.penalizer*np.eye(d + self.fit_intercept)
+        
+        t_0 = sorted_event_times[0]
+        cum_v = np.zeros((d+self.fit_intercept,1))
+        v = cum_v.copy()
+        for i,time in enumerate(sorted_event_times):
+            relevant_times = (t_0<timeline)*(timeline<=time)
+            if observed[i] == 0:
+                X_[i,:] = 0
+            try:
+                V = dot(inv(dot(X_.T,X_) + penalizer), X_.T)
+            except LinAlgError:
+                #if penalizer > 0, this should not occur. But sometimes it does...
+                V = dot(pinv(dot(X_.T,X_) + penalizer), X_.T)
+
+            v = dot(V, basis(n,i))
+            cum_v = cum_v + v
+            self.cumulative_hazards_.ix[relevant_times] = self.cumulative_hazards_.ix[relevant_times].values + cum_v.T
+            self.hazards_.iloc[i] = self.hazards_.iloc[i].values + v.T
+            self._variance.ix[relevant_times] = self._variance.ix[relevant_times].values + dot( V[:,i][:,None], V[:,i][None,:] ).diagonal()
+            t_0 = time
+            X_[i,:] = 0
+
+        #clean up last iteration
+        relevant_times = (timeline>time)
+        self.cumulative_hazards_.ix[relevant_times] = cum_v.T
+        self.hazards_.iloc[i] = v.T
+        self._variance.ix[relevant_times] = dot( V[:,i][:,None], V[:,i][None,:] ).diagonal()
+        self.timeline = timeline
+        self.X = X
+        self.censorship = censorship
+        self.event_times = event_times
+        self._compute_confidence_intervals()
+        return self
+
+    def smoothed_hazards_(self, bandwith=1):
+        """
+        Using the epanechnikov kernel to smooth the hazard function, with sigma/bandwith
 
         """
         C = self.censorship.astype(bool)
-        return pd.DataFrame( np.dot(gaussian(self.timeline[:,None], self.timeline[C][None,:],bandwith), self.hazards_.values[C,:]), 
+        return pd.DataFrame( np.dot(epanechnikov_kernel(self.timeline[:,None], self.timeline[C],bandwith), self.hazards_.values[C,:]), 
                 columns=self.hazards_.columns, index=self.timeline)
 
-  def _compute_confidence_intervals(self):
+    def _compute_confidence_intervals(self):
         alpha2 = inv_normal_cdf(1 - (1-self.alpha)/2)
         n = self.timeline.shape[0]
         d = self.cumulative_hazards_.shape[1]
@@ -303,41 +348,41 @@ class AalenAdditiveFitter(object):
         self.confidence_intervals_.ix['lower'] = self.cumulative_hazards_.values - alpha2*np.sqrt(self._variance.values)
         return 
 
-  def predict_cumulative_hazard(self, X, columns=None):
-    """
-    X: a (n,d) covariate matrix
+    def predict_cumulative_hazard(self, X, columns=None):
+        """
+        X: a (n,d) covariate matrix
 
-    Returns the hazard rates for the individuals
-    """
-    n,d = X.shape
-    try:
-      X_ = X.values.copy()
-    except:
-      X_ = X.copy()
-    X_ = X.copy() if not self.fit_intercept else np.c_[ X.copy(), np.ones((n,1)) ]
-    return pd.DataFrame(np.dot(self.cumulative_hazards_, X_.T), index=self.timeline, columns=columns )
-  
-  def predict_survival_function(self,X, columns=None):
-    """
-    X: a (n,d) covariate matrix
+        Returns the hazard rates for the individuals
+        """
+        n,d = X.shape
+        try:
+          X_ = X.values.copy()
+        except:
+          X_ = X.copy()
+        X_ = X.copy() if not self.fit_intercept else np.c_[ X.copy(), np.ones((n,1)) ]
+        return pd.DataFrame(np.dot(self.cumulative_hazards_, X_.T), index=self.timeline, columns=columns )
+      
+    def predict_survival_function(self,X, columns=None):
+        """
+        X: a (n,d) covariate matrix
 
-    Returns the survival functions for the individuals
-    """
-    return np.exp(-self.predict_cumulative_hazard(X, columns=columns))
+        Returns the survival functions for the individuals
+        """
+        return np.exp(-self.predict_cumulative_hazard(X, columns=columns))
 
-  def predict_median(self,X):
-    """
-    X: a (n,d) covariate matrix
-    Returns the median lifetimes for the individuals
-    """
-    return median_survival_times(self.predict_survival_function(X))
+    def predict_median(self,X):
+        """
+        X: a (n,d) covariate matrix
+        Returns the median lifetimes for the individuals
+        """
+        return median_survival_times(self.predict_survival_function(X))
 
-  def predict_expectation(self,X):
-    """
-    Compute the expected lifetime, E[T], using covarites X.
-    """
-    t = self.cumulative_hazards_.index
-    return quadrature(self.predict_survival_function(X).values.T, t)
+    def predict_expectation(self,X):
+        """
+        Compute the expected lifetime, E[T], using covarites X.
+        """
+        t = self.cumulative_hazards_.index
+        return quadrature(self.predict_survival_function(X).values.T, t)
 
 #utils
 def qth_survival_times(q, survival_functions):
@@ -367,11 +412,8 @@ def qth_survival_times(q, survival_functions):
 def median_survival_times(survival_functions):
     return qth_survival_times(0.5, survival_functions)
 
-def gaussian(t,T,sigma=1.):
-    return 1./np.sqrt(np.pi*2.*sigma**2)*np.exp(-0.5*(t-T)**2/sigma**2)
-
-def ipcw(target_event_times, target_censorship, predicted_event_times ):
-    pass
+def asymmetric_epanechnikov_kernel(q, x):
+    return (64*(2 - 4*q + 6*q*q - 3*q**3) + 240*(1-q)**2*x)/((1+q)**4*(19 - 18*q + 3*q**2))
 
 """
 References:
